<?php

namespace TekNutri\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Doctrine\DBAL\Connection;
use Exception;
use Zeedhi\Framework\DTO\Response\Error;

class GeraLiberacao {

    protected $connection;
    protected $util;
    protected $environment;
    protected $nrlista;

    public function __construct($connection, $util, $environment) {
        $this->connection    = $connection;
        $this->util          = $util;
        $this->environment   = $environment;
    }

    public function geraLiberacao($request, $response) {
       try{
            $row = $request->getRow(); 
            $produtosComNutr = [];
            $produtosSemNutr = [];
            
            //Cria lista que retorna array com os produtos e se possuem nutrientes ou não           
            $this->nrlista = $this->util->insertLista($row['CDPRODUTOS']);
            $row['NRLISTA'] = $this->nrlista;
            $verificaProdutos = "SELECT P.CDPRODUTO, P.NMPRODUTO,
                                DECODE(MAX(N.CDNUTRIENTE), NULL, 'N', 'S') AS TEMNUTRIENTE, P.NRORG
                                FROM PRODUTO P
                                INNER JOIN LISTA L ON L.NRLISTA = :P_NRLISTA AND L.CDLISTA = P.CDPRODUTO AND L.NRORG = P.NRORG
                                LEFT JOIN NUTRPROD N ON P.CDPRODUTO = N.CDPRODUTO AND P.NRORG = N.NRORG
                                WHERE P.NRORG = :P_NRORG
                                GROUP BY P.CDPRODUTO, P.NMPRODUTO, P.NRORG"; 
            $stmt = $this->connection->prepare($verificaProdutos);
            $params = array(
                "P_NRLISTA" => $row['NRLISTA'],
                "P_NRORG" => $this->environment->getNrOrg()
            );
            $stmt->execute($params);
            $produtos = $stmt->fetchAll(\PDO::FETCH_ASSOC);
            
            //Separa os produtos que têm nutrientes dos que não têm
            forEach($produtos as &$produto){
                if($produto['TEMNUTRIENTE'] == 'S'){
                    array_push($produtosComNutr, $produto);
                } else if($produto['TEMNUTRIENTE'] == 'N'){
                    array_push($produtosSemNutr, $produto);
                }
            }
            unset($produto);
            
            //Libera os produtos que contêm nutrientes
            $this->liberaProdutos($produtosComNutr);
            //Verifica se existem produtos com o nutrientes para montar a mensagem
            if(sizeof($produtosSemNutr) != 0){
                $mensagem = $this->montaMensagem($produtosSemNutr);
            } else {
                $mensagem = ["Os produtos foram liberados com sucesso."];
            }
            $response->addDataSet(new DataSet("mensagem", $mensagem));
            
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        } finally {
            $this->util->deleteLista($this->nrlista);
        }
   }
        
    public function liberaProdutos($produtosComNutr){
        $cdProdutos = "";
        forEach($produtosComNutr as &$produto){
            $cdProdutos = $cdProdutos.$produto['CDPRODUTO']."_";
        }
        $produtos = "UPDATE SIT_PRODUTOINT
                        SET DTLIBERACAO = SYSDATE,
                            DTENVIO = '',
                            DSOBSINT = '',
                            CDOPERULTATU =:CDOPERULTATU,
                            DTULTATU = SYSDATE
                      WHERE INSTR(:CDPRODUTO, CDPRODUTO) > 0
                        AND QTPESOINT > 0";        
        $stmt = $this->connection->prepare($produtos);
        
        $params = array(
            "CDPRODUTO" => $cdProdutos,
            "CDOPERULTATU" => $this->environment->getCdOperador()
        );
        
        $stmt->execute($params);
    }   
    
    public function montaMensagem($produtosSemNutr){
        $mensagem = "";
		for($i = 0; $i < sizeof($produtosSemNutr); $i++){
			if($i != sizeof($produtosSemNutr) - 1){
				$mensagem = $mensagem.$produtosSemNutr[$i]['NMPRODUTO'].", ";
			}else{
				$mensagem = $mensagem.$produtosSemNutr[$i]['NMPRODUTO'];
			}
		}
		$mensagem = ["Liberação realizada com sucesso, porém alguns produtos não foram liberados: ".$mensagem."."];
		return $mensagem;
    }
}
